/*********************************************************************************
 *  Samurai Kanji, version 1.0                                                   *
 *  Copyright (C) 2006 Robert Baruch                                             *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  Please see the software license located at /license.txt for more details.    *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Robert Baruch                                                             *
 *     76 Collins Lane                                                           *
 *     Rising Sun, MD  21911                                                     *
 *     USA                                                                       *
 *     E-mail: autophile@gmail.com                                               *
 *********************************************************************************/

import waba.io.*;
import waba.sys.*;
import waba.util.*;
import waba.fx.*;

public class KanjiDatabase
{
    KanjiData[] kanjiData;
    IntHashtable lessonLengths = new IntHashtable(20);
    IntHashtable lessonIndices = new IntHashtable(20);

    static class KanjiData
    {
        int lesson;
        String meaning;
        int recordNum;
        int recordOffset;
        int compressedDataSize;
    }

    public KanjiDatabase()
    {
        Catalog db = new Catalog("SamuraiKanjiData.CHFY.data", Catalog.READ_ONLY);

        // Surely we can find a better way of notifying the user
        // of a fatal error than this! Like... a MessageBox?

        if (!db.isOpen())
        {
            Vm.debug("Error: Couldn't find SamuraiKanjiData database");
            return;
        }

        Vm.debug("Opened SamuraiKanjiData database");

        DataStream in = new DataStream(db);

        // Read the kanji information from the DB, but do not read
        // the PNG file for each kanji -- just store the offset
        // and length of the file.
        //
        // Also note that all kanji for a given lesson must be
        // consecutive, so that we can say where a given lesson
        // begins and ends in the database.

        Vector records = new Vector();
        int index = 0;
        int recordNum = 0;

        // Note that there is a limit of about 65000 bytes to each
        // Palm record. Since the kanji data exceeds this limit,
        // kanji records are split among multiple DB records.

        while (db.setRecordPos(recordNum)) // scan through the DB records
        {
            int dataSize = db.getRecordSize();

            while (dataSize > 0) // parse out each kanji record
            {
                KanjiData kanji = new KanjiData();
                kanji.lesson = ((int)in.readByte())&0xFF; // signed to unsigned!
                kanji.compressedDataSize = ((int)in.readShort())&0xFFFF;
                kanji.meaning = in.readCString();
                kanji.recordNum = recordNum;
                kanji.recordOffset = db.getRecordOffset();

                in.skip(kanji.compressedDataSize); // skip over the PNG data
                records.add(kanji);

                dataSize -= (9 + kanji.meaning.length() + kanji.compressedDataSize);

                Vm.debug("Meaning " + kanji.meaning +
                         " record " + recordNum +
                         " remaining bytes: " + dataSize +
                         " offset: " + kanji.recordOffset);

                // Add one to the lesson length. If there isn't any info
                // for this lesson, note the current kanji index as the
                // start of the lesson, and set the lesson length to 1.

                int lessonLength = lessonLengths.get(kanji.lesson);
                if (lessonLength == IntHashtable.INVALID)
                {
                    lessonLength = 0;
                    lessonIndices.put(kanji.lesson, index);
                }
                lessonLength++;
                lessonLengths.put(kanji.lesson, lessonLength);

                index++;
            }

            db.setRecordPos(-1); // unlock the record
            recordNum++;
        }

        kanjiData = new KanjiData[records.size()];
        for (int i=0; i<kanjiData.length; i++)
            kanjiData[i] = (KanjiData)records.items[i];

        db.close();

        Vm.debug("Read " + kanjiData.length + " kanji");
    }

    public Image toImage(int lesson, int kanjiIndex)
    {
        int index = getKanjiIndex(lesson, kanjiIndex);
        if (index == IntHashtable.INVALID)
            return null;
        return toImage(index);
    }

    public Image toImage(int kanjiIndex)
    {
        Bmp bitmap = new Bmp(); // to store the uncompressed data in BMP file format
        KanjiData kanji = kanjiData[kanjiIndex];

        Catalog db = new Catalog("SamuraiKanjiData.CHFY.data", Catalog.READ_ONLY);

        // We currently don't handle the fatal error in the constructor
        // if we don't find the DB. Otherwise this wouldn't be necessary.

        if (!db.isOpen())
        {
            Vm.debug("Error, DB couldn't be opened");
            return null;
        }

        DataStream in = new DataStream(db);

        if (!in.isOpen())
        {
            Vm.debug("Error, data stream couldn't be opened");
            db.close();
            return null;
        }

        db.setRecordPos(kanji.recordNum);
        in.skip(kanji.recordOffset);

        // read the PNG data

        byte[] png = new byte[kanji.compressedDataSize];
        in.readBytes(png);

        db.close();

        // Note that we already know what the image format is, so there
        // is no need to read the header. Unless you're paranoid.
        // And I'm not sufficiently paranoid yet.

        // Find the IDAT chunk, read the data length
        //
        // All chunks have this format:
        //
        // chunk name (4 bytes)
        // length of data (4 bytes)
        // data
        // CRC (4 bytes)
        //
        // Technically, the data could be spread out over multiple
        // IDAT chunks, but we're playing fast and loose here, assuming
        // that the original PNG generator emitted only one IDAT chunk.

        if (png.length < 20) // can't possibly be a PNG file
        {
            Vm.debug("PNG file length is too small (" + png.length + ")");
            return null;
        }

        // Make sure header is a PNG header: 137 80 78 71 13 10 26 10

        if (png[0] != (byte)137 || png[1] != 80 || png[2] != 78 || png[3] != 71 ||
            png[4] != 13 || png[5] != 10 || png[6] != 26 || png[7] != 10)
        {
            Vm.debug("PNG file does not have PNG signature");
            return null;
        }

        int pos = 8;
        int chunklen = 0;

        // Go through the chunks to find the first (and, we assume, only)
        // IDAT chunk

        while (pos < png.length-8)
        {
            chunklen = ((int)png[pos]<<24)&0xFF000000;
            chunklen |= ((int)png[pos+1]<<16)&0x00FF0000;
            chunklen |= ((int)png[pos+2]<<8)&0x0000FF00;
            chunklen |= ((int)png[pos+3])&0x000000FF;
            if (png[pos+4] == 'I' &&
                png[pos+5] == 'D' &&
                png[pos+6] == 'A' &&
                png[pos+7] == 'T')
                break;
           pos += 8 + chunklen + 4; // + 4 bytes CRC
        }

        if (pos >= png.length-8)
        {
            Vm.debug("PNG file does not contain image data");
            return null;
        }

        // hooray, we found the IDAT chunk. Read it into an array

        pos += 8;
        if (pos + chunklen > kanji.compressedDataSize)
        {
            // This should never happen unless the DB was
            // not written correctly.

            Vm.debug("PNG image data is cut off (chunk is short, expected " + chunklen + " bytes)");
            return null;
        }

        ByteArrayStream compressedArray = new ByteArrayStream(png);
        compressedArray.skipBytes(pos);

        // inflate compressed data

        // When available in next release:
        // superwaba.ext.xplat.zlib.ZLib.inflate(compressedArray, bitmap);

        com.jcraft.jzlib.ZInputStream zin =
            new com.jcraft.jzlib.ZInputStream(compressedArray);

        // The first byte of each line is the filter
        // byte. We are going to assume the filter byte is zero
        // (meaning no filtering).

        byte[] lineData = new byte[128];

        for (int line = 0; line < 128; line++)
        {
            int filter = zin.read();
            if (filter == -1) // skip one
            {
                Vm.debug("PNG image data is cut off (couldn't read filter byte at line " + line + ")");
                return null;
            }
            if (filter != 0) // we don't handle filtered data
            {
                Vm.debug("PNG image data is bad (can't handle filtered line at line " + line + ")");
                return null;
            }
            if (zin.readBytes(lineData, 0, 128) != 128)
            {
                Vm.debug("PNG image data is cut off (couldn't read line at line " + line + ")");
                return null;
            }
            bitmap.writeBytes(lineData);
        }

        // and finally convert to an image


        if (SamuraiKanji.screenWidth == 320)
            return bitmap.toImage();
        return bitmap.toImage().getScaledInstance(64, 64);
    }

    public String toMeaning(int kanji)
    {
        return kanjiData[kanji].meaning;
    }

    public String toMeaning(int lesson, int kanjiIndex)
    {
        int index = getKanjiIndex(lesson, kanjiIndex);
        if (index == IntHashtable.INVALID)
            return null;
        return kanjiData[index].meaning;
    }

    public int getLessonLength(int lesson)
    {
        return lessonLengths.get(lesson);
    }

    public int getKanjiIndex(int lesson, int kanjiIndex)
    {
        int startIndex = lessonIndices.get(lesson);
        if (startIndex == IntHashtable.INVALID)
            return IntHashtable.INVALID;
        int lessonLength = lessonLengths.get(lesson);
        if (lessonLength == IntHashtable.INVALID)
            return IntHashtable.INVALID;
        if (kanjiIndex >= lessonLength)
        {
            Vm.debug("Kanji index " + kanjiIndex + " given in lesson " +
                     lesson + " is out of range (only " + lessonLength +
                     " kanji in lesson)");
            return IntHashtable.INVALID;
        }
        return startIndex + kanjiIndex;
    }

    public int getNumLessons()
    {
        return lessonIndices.size();
    }
}
