/*********************************************************************************
 *  Samurai Kanji, version 1.0                                                   *
 *  Copyright (C) 2006 Robert Baruch                                             *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  Please see the software license located at /license.txt for more details.    *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Robert Baruch                                                             *
 *     76 Collins Lane                                                           *
 *     Rising Sun, MD  21911                                                     *
 *     USA                                                                       *
 *     E-mail: autophile@gmail.com                                               *
 *********************************************************************************/

import waba.ui.*;
import waba.fx.*;
import waba.sys.*;

public class LessonSelectionWindow extends Window
{
    PushButtonGroup buttons;
    Image nowShowing = null;
    Check[] checks;
    ScrollBar scrollBar;
    int totalChecks;
    Label totalKanji;

    public boolean[] selected;
    public boolean cancelled = false;

    public LessonSelectionWindow()
    {
        super("Select Lessons to Include", RECT_BORDER);

        // add a scroll bar on the left

        scrollBar = new ScrollBar(ScrollBar.VERTICAL);
        add(scrollBar);
        scrollBar.setRect(LEFT + 2, TOP, PREFERRED, FILL);

        // compute the number of checks we can fit in the screen,
        // and the top margin

        int windowHeight = getClientRect().height;
        Check check0 = new Check("Lesson 1");
        int checkHeight = check0.getPreferredHeight() + 2;
        totalChecks = windowHeight / checkHeight;
        int margin = (windowHeight - totalChecks*checkHeight)/2;

        // put in all the checks along the left side of the screen
        // to the right of the scroll bar

        checks = new Check[totalChecks];
        checks[0] = check0;
        add(checks[0], AFTER + 5, TOP + margin);

        for (int i=1; i<checks.length; i++)
            add(checks[i] = new Check("Lesson " + (i+1)), SAME, AFTER + 2);

        // put in the buttons on the upper right of the screen

        buttons = new PushButtonGroup(
            new String[] { "Select all", "Clear all", "Accept changes", "Cancel changes" },
            false,  // at least one stays selected?
            -1,     // default selected
            2,      // gap
            6,      // inside gap
            4,      // number of rows
            true,   // all same width?
            PushButtonGroup.BUTTON // type of buttons
            );

        add(buttons, RIGHT - 2, TOP + 2);

        // kanji count label on the lower right

        totalKanji = new Label("Kanji: 0000");
        add(totalKanji, RIGHT - 2, BOTTOM - 2);

        // set the scrollbar limits. We may not have enough lessons,
        // so we may have more checks than lessons. In that case,
        // make sure that the checks that don't represent lessons
        // are disabled.

        int totalLessons = SamuraiKanji.kanji.getNumLessons();

        scrollBar.setMinimum(0);
        scrollBar.setMaximum(
            totalChecks > totalLessons ? totalChecks : totalLessons);
        scrollBar.setVisibleItems(totalChecks);


        selected = new boolean[scrollBar.getMaximum()];
        for (int i=0; i<SamuraiKanji.selectedLessons.size(); i++)
        {
            int item = SamuraiKanji.selectedLessons.items[i] - 1;
            selected[item] = true;
            if (item < checks.length)
                checks[item].setChecked(true);
        }

        if (totalLessons < totalChecks)
            for (int i=totalLessons; i<totalChecks; i++)
                checks[i].setEnabled(false);

        // count and display the number of kanji currently selected

        countSelectedKanji();
    }

    int countSelectedKanji()
    {
        int total = 0;
        int totalLessons = SamuraiKanji.kanji.getNumLessons();

        for (int i=0; i<selected.length; i++)
            if (selected[i] && i < totalLessons)
                total += SamuraiKanji.kanji.getLessonLength(i+1);
        totalKanji.setText("Kanji: " + total);
        return total;
    }

    public void onPaint(Graphics g)
    {
        super.onPaint(g);
    }

    public void onEvent(Event e)
    {
        if (e.type == ControlEvent.PRESSED && e.target == scrollBar)
        {
            e.consumed = true;
            int start = scrollBar.getValue();
            Graphics g = createGraphics();
            g.setBackColor(this.getBackColor());
            for (int i=0; i<totalChecks; i++)
            {
                checks[i].setText("Lesson " + (1 + i + start));
                checks[i].setChecked(selected[start + i]);

                // I might not need to do all this blanking stuff if
                // I had initialized the checks with "Lesson 0000" to
                // set the maximum possible extent for each check.
                // But that's an experiment for another day.

                Rect r = checks[i].getRect();
                int prefWidth = checks[i].getPreferredWidth();
                if (r.width > prefWidth) // blank out area that used to contain text
                {
                    g.fillRect(r.x, r.y, r.width, r.height);
                }
                r.width = prefWidth; // to change width for new label
                checks[i].setRect(r);
            }
            g.free();
            return;
        }
        if (e.type == ControlEvent.PRESSED && e.target == buttons)
        {
            int index = buttons.getSelected();

            switch (index)
            {
            case 0: // select all
                for (int i = 0; i < checks.length; i++)
                    checks[i].setChecked(true);
                for (int i = 0; i < selected.length; i++)
                    selected[i] = true;
                break;

            case 1: // clear all
                for (int i = 0; i < checks.length; i++)
                    checks[i].setChecked(false);
                for (int i = 0; i < selected.length; i++)
                    selected[i] = false;
                break;

            case 2: // accept changes
            {
                boolean all = false;

                for (int i=0; i<selected.length; i++)
                    if (selected[i])
                    {
                        all = true;
                        break;
                    }

                if (!all)
                {
                    popupBlockingModal(new MessageBox(
                            "Hey!",
                            Convert.insertLineBreak(
                                    Settings.screenWidth - 6,
                                    '|',
                                    getFontMetrics(getFont()),
                                    "You must select at least one lesson. " +
                                    "Try again, or use 'Cancel changes'.")));
                    break;
                }
                unpop();
                break;
            }

            case 3: // cancel changes
                cancelled = true;
                unpop();
                break;
            }

            countSelectedKanji(); // total them up

            e.consumed = true;
            return;
        }

        if (e.type == ControlEvent.PRESSED)
        {
           for (int i=0; i<checks.length; i++)
               if (e.target == checks[i])
               {
                   selected[i+scrollBar.getValue()] = checks[i].getChecked();
                   countSelectedKanji();
                   e.consumed = true;
                   return;
               }
        }

    }
}
