/*********************************************************************************
 *  Samurai Kanji, version 1.0                                                   *
 *  Copyright (C) 2006 Robert Baruch                                             *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  Please see the software license located at /license.txt for more details.    *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Robert Baruch                                                             *
 *     76 Collins Lane                                                           *
 *     Rising Sun, MD  21911                                                     *
 *     USA                                                                       *
 *     E-mail: autophile@gmail.com                                               *
 *********************************************************************************/

import waba.ui.*;
import waba.fx.*;
import waba.sys.*;
import waba.util.*;

public class SamuraiKanji extends MainWindow {
    static Image img;
    public static int screenWidth;
    public static int screenHeight;
    public static KanjiDatabase kanji;
    static MenuBar menu;
    static boolean showingFront = true;
    static boolean kanjiOnFront = true;
    static Font font;
    static IntVector selectedLessons = new IntVector(20);
    static IntVector randomizedKanji = new IntVector(500);
    static int showingKanji;
    static Button flipCard;
    static Button nextCard;
    static String[][] menuItems;

    // A list of images used by the application (not kanji images)
    public static Image[] buttonImages = new Image[5];

    public final static int ARROW_RIGHT = 0;
    public final static int ARROW_LEFT = 1;
    public final static int NEXT = 2;
    public final static int ABOUT1 = 3;
    public final static int ABOUT2 = 4;

    public SamuraiKanji()
    {
        super("Samurai Kanji", TAB_ONLY_BORDER);
        waba.sys.Settings.setUIStyle(waba.sys.Settings.PalmOS);
    }

    public void onStart()
    {
        screenWidth = waba.sys.Settings.screenWidth;
        screenHeight = waba.sys.Settings.screenHeight;
        Vm.debug("Starting app on screen " + screenWidth + "x" + screenHeight);
        kanji = new KanjiDatabase();
        Vm.debug("Kanji database loaded");

        String[] menuStringsHelp = { "Help", "About Samurai Kanji" };
        String[] menuStringsOptions = { "Options", "English on Top", "Lesson Selection", "Reshuffle" };

        menuItems = new String[][] { menuStringsOptions, menuStringsHelp };
        menu = new MenuBar(menuItems);
        setMenuBar(menu);

        // select all lessons by default

        for (int i=0; i<kanji.getNumLessons(); i++)
            selectedLessons.add(i+1);

        font = new Font("Default", Font.BOLD, 20);

        if (screenWidth == 320)
        {
            buttonImages[ARROW_RIGHT] = new Image("arrow-right-32x32.bmp");
            buttonImages[ARROW_LEFT] = new Image("arrow-left-32x32.bmp");
            buttonImages[NEXT] = new Image("next-32x32.bmp");
            buttonImages[ABOUT1] = new Image("about-8-d144.bmp");
            buttonImages[ABOUT2] = new Image("about2-8-d144.bmp");
        }
        else
        {
            buttonImages[ARROW_RIGHT] = new Image("arrow-right-16x16.bmp");
            buttonImages[ARROW_LEFT] = new Image("arrow-left-16x16.bmp");
            buttonImages[NEXT] = new Image("next-16x16.bmp");
            buttonImages[ABOUT1] = new Image("about-8.bmp");
            buttonImages[ABOUT2] = new Image("about2-8.bmp");
        }

        flipCard = new Button(buttonImages[ARROW_RIGHT]);
        add(flipCard, Control.RIGHT-10, Control.BOTTOM-10);

        nextCard = new Button(buttonImages[NEXT]);
        add(nextCard, Control.SAME, Control.BEFORE, flipCard);

        reshuffle();
    }

    public void onPaint(Graphics g)
    {
        super.onPaint(g);

        try
        {
            if ((showingFront && kanjiOnFront) || (!showingFront && !kanjiOnFront)) {
                img = kanji.toImage(randomizedKanji.items[showingKanji]);
                g.drawImage(img, CENTER, CENTER);
            } else {
                g.setForeColor(Color.BLACK);
                g.setFont(font);
                FontMetrics fm = this.getFontMetrics(font);
                String meaning = kanji.toMeaning(randomizedKanji.items[showingKanji]);
                int w = fm.getTextWidth(meaning);
                int h = fm.height;
                g.drawText(meaning,
                           (screenWidth - w) / 2,
                           (screenHeight - h) / 2);
            }
        }
        catch (Exception ex)
        {
            Vm.debug("Exception in paint routine:");
            ex.printStackTrace();
        }
    }

    void randomizeKanji()
    {
        Random random = new Random();
        randomizedKanji.clear();
        for (int i=selectedLessons.size()-1; i>=0; i--)
        {
            int lesson = selectedLessons.items[i];
            int lessonLength = kanji.getLessonLength(lesson);
            for (int k=0; k < lessonLength; k++)
                randomizedKanji.add(kanji.getKanjiIndex(lesson, k));
        }

        // randomly switch indices

        int size = randomizedKanji.size();
        for (int i=0; i<size; i++)
        {
            int pos1 = random.nextInt(size);
            int pos2 = random.nextInt(size);
            int tmp = randomizedKanji.items[pos1];
            randomizedKanji.items[pos1] = randomizedKanji.items[pos2];
            randomizedKanji.items[pos2] = tmp;
        }
    }

    void flipCardOver(Event e)
    {
        if (img != null)
           img.free();
       showingFront = !showingFront;
       flipCard.setImage(buttonImages[showingFront ? ARROW_RIGHT : ARROW_LEFT]);
       e.consumed = true;
       repaint();
    }

    void gotoNextCard(Event e)
    {
        if (img != null)
            img.free();
        showingFront = true;
        flipCard.setImage(buttonImages[ARROW_RIGHT]);

        showingKanji = (showingKanji + 1) % randomizedKanji.size();

        e.consumed = true;
        repaint();
    }

    void reshuffle()
    {
        randomizeKanji();
        showingKanji = 0;
        showingFront = true;
        repaint();
    }

    public void onEvent(Event e)
    {
        if (e.type == ControlEvent.WINDOW_CLOSED && e.target == menu)
        {
            if (menu.getSelectedMenuItem() == 101) // Help... About
            {
                popupBlockingModal(new AboutWindow());
            }
            else if (menu.getSelectedMenuItem() == 1) // Change what's on top
            {
                kanjiOnFront = !kanjiOnFront;
                if (kanjiOnFront)
                    menuItems[0][1] = "English on Top";
                else
                    menuItems[0][1] = "Kanji on Top";
                menu.setItems(menuItems);
                repaint();
            }
            else if (menu.getSelectedMenuItem() == 2) // Select lessons
            {
                LessonSelectionWindow selections = new LessonSelectionWindow();
                popupBlockingModal(selections);
                if (!selections.cancelled)
                {
                    selectedLessons.clear();
                    for (int i=0; i<selections.selected.length; i++)
                        if (selections.selected[i])
                            selectedLessons.add(i+1);
                    reshuffle();
                }
            }
            else if (menu.getSelectedMenuItem() == 3) // Reshuffle
            {
                reshuffle();
            }
            e.consumed = true;
            return;
        }

        if (e.type == ControlEvent.PRESSED && e.target == flipCard)
        {
            flipCardOver(e);
            return;
        }

        if (e.type == ControlEvent.PRESSED && e.target == nextCard)
        {
            gotoNextCard(e);
            return;
        }

        /*
        if (e.type == PenEvent.PEN_UP && e.target == this) // tap the screen
        {
            flipCardOver(e);
            return;
        }
        */

        if (e.type == KeyEvent.KEY_PRESS)
        {
            if ( showingFront && ((KeyEvent)e).isDownKey() )
            {
                flipCardOver(e);
                return;
            }

            if ( !showingFront && ((KeyEvent)e).isUpKey() )
            {
                flipCardOver(e);
                return;
            }

            if ( !showingFront && ((KeyEvent)e).isNextKey() )
            {
                gotoNextCard(e);
                return;
            }
        }
    }

    public void onExit() {

    }
}
